% Program to plot stability plots for predictor-corrector methods

% A linear multistep formula is: x+ = G(w) x  with w = a*Delta t
% for differential equation xdot = ax with stepsize Delta t
%
% The eigenvalues of G determine the stability of the method;
% The method is absolutely stable iff max (abs( eig G(w))) < 1

% We find the value of w s.t. z = exp(i*theta) is an eigenvalue
% of G for theta in [0, 2*pi].  Actually, [0, 4*pi] to cover entire
% curve in w.

% We demonstrate here stability curve for APC2, APC3 and APC4 methods.
% NOTE: APCn uses an n-1st order predictor and nth order corrector
%

% Setting grid points

% pass parameters to function, jbr, 6/28/2018

axisbox = [-1.5 1.5 -1 1];
xa = axisbox(1); xb = axisbox(2); ya = axisbox(3); yb = axisbox(4);
nhalf  = 50;
npts = 2*nhalf+1;
theta = linspace(0, 4*pi, npts);
lamvec = exp(i*theta);

% predictor coefficients, 1st, 2nd, 3rd, 4th order
p{1} = [1, 0, 0, 0];
p{2} = [3/2, -1/2, 0, 0];
p{3} = [23/12, -4/3, 5/12, 0];
p{4} = 1/24*[55, -59, 37, -9];
% corrector coefficients, 1st, 2nd, 3rd, 4th order
c = cell;
c{1} = [1, 0, 0, 0];
c{2} = [1/2, 1/2, 0, 0];
c{3} = 1/12*[5, 8, -1, 0];
c{4} = 1/24*[9, 19, -5, 1];

par = struct();
par.p = p; par.c = c;


PCsvec = [2,3,4];
npcs = length(PCsvec);
wsol(1:npcs) = {zeros(npts,1)};
optimset("ComplexEqn", "on");
for j = 1:npcs
  PCs = PCsvec(j);
  par.PCs = PCs;
  w0 = 0 + 0*i;
  for k = 1:length(theta)
    lam = lamvec(k);
    par.lam = lam;
    [w,fval,info] = fsolve(@(w) characteristic(w, par), w0);
    if (info <= 0 )
      j, k, info, w, w0
    end
    w0 = w;
    wsol{j}(k) = w;
  end
end

% check for intersecting loops
for ind = 1:npcs
  for k = 1: length(wsol{ind})-1
    z = [real(wsol{ind})'; imag(wsol{ind})'];
    iloop = 0;
    for j = k+2 : length(wsol{ind})-1
      % find first place where z(k) intersects the rest of the path
      lam = inv([z(:,k)-z(:,k+1), z(:,j+1)-z(:,j)])*(z(:,j+1)-z(:,k+1));
      if (lam >=0 && lam <=1)
	iloop = k+1:j;
	zint = lam(1)*z(:,k)+(1-lam(1))*z(:,k+1);
	break
      end
    end
    if (iloop ~= 0)
      % chop out wsol(iloop) and replace with single interpolated value zint
      zcp = complex(zint(1),zint(2));
      wsol{ind}(iloop(1)) = zcp;
      iloop(1) = [];
      wsol{ind}(iloop) = [];
    end
  end
end

figure()
hold on
for j = 1:npcs
  plot(real(wsol{j}), imag(wsol{j}),'-o')
end
%axis (axisbox)

% rewrite real and imaginary parts for plotting

fopen("APC_stability.dat", "w+");
for k = 1: npcs
  tmp = cell2mat(wsol(k));
  store = [real(tmp), imag(tmp)];
  save ("APC_stability.dat", "store", "-append")
end